@@ -2,14 +2,12 @@ package HTML::TurboForm;
 
 use strict;
 use warnings;
-
 use UNIVERSAL::require;
 use YAML::Syck;
-
-our $VERSION='0.26';
+our $VERSION='0.48';
 
 sub new{
-  my ($class, $r)=@_;
+  my ($class, $r,$prefix)=@_;
   my $self = {};
   $self->{request}= $r;
   $self->{submitted} = 0;
@@ -17,6 +15,8 @@ sub new{
   $self->{count}=0;
   $self->{submit_id} = -1;
   $self->{addition_modules}='';
+  $self->{prefix}='';
+  $self->{prefix}=$prefix if ($prefix);
 
   bless( $self, $class );
   return $self;
@@ -30,7 +30,7 @@ sub add_modules{
 sub add_constraint{
   my ($self, $params) = @_;
 
-  my $name= $params->{name};
+  my $name= $self->{prefix}.$params->{name};
   $params->{request}=$self->{request};
   my $class_name = "HTML::TurboForm::Constraint::" . $params->{ type };
   $class_name->require() or die "Constraint Class '" . $class_name . "' does not exist: $@";
@@ -58,6 +58,15 @@ sub load{
     }
 }
 
+sub unignore_all{
+  my ($self ) = @_;
+  my $k;
+  my $v;
+  foreach $k(keys %{ $self->{element_index} } ){
+    $self->{element_index}->{$k}->{ignore}='false';
+  }
+}
+
 sub ignore_all{
   my ($self ) = @_;
 
@@ -68,13 +77,22 @@ sub ignore_all{
   }
 }
 
+sub remove_all{
+  my ($self ) = @_;
+
+  $self->{element_index}={};
+  $self->{element}=();
+}
+
 sub ignore_element{
   my ($self, $name ) = @_;
+  $name=$self->{prefix}.$name;
   $self->{element_index}->{$name}->{ignore}='true';
 }
 
 sub unignore_element{
   my ($self, $name ) = @_;
+  $name=$self->{prefix}.$name;
   $self->{element_index}->{$name}->{ignore}='false';
 }
 
@@ -87,8 +105,11 @@ sub add_element{
     $self->{count}++;
   }
   $params->{request}=$self->{request};
+  my $namew= $params->{name};
+  my $name= $self->{prefix}.$params->{name};
+  $params->{name}=$name;
+  #print $name."\n";
 
-  my $name= $params->{name};
   my $class_name = "HTML::TurboForm::Element::" . $params->{ type };
   $class_name->require() or die "Class '" . $class_name . "' does not exist: $@";
   my $element= $class_name->new($params,$self->{uploads}->{$name.'_upload'});
@@ -101,23 +122,41 @@ sub add_element{
   $self->{element_index}->{$name}->{error_message}='';
 
   if ($params->{type} eq 'Submit') {
-    if ( exists $self->{request}->{$name } ){
+    if (( exists $self->{request}->{$name.".x"} )or(exists $self->{request}->{$name})){
       $self->{submitted}=1 ;
-      $self->{submit_value} = $name;
+      $self->{submit_value} = $namew;
     }
   }
 
   if ($params->{submit}){
     if ( $self->{request}->{$name} ){
       $self->{submitted}=1 ;
-      $self->{submit_value} = $name;
+      $self->{submit_value} = $namew;
     }
   }
 
   if ($params->{type} eq 'Image') {
     if ( exists $self->{request}->{$name.'_submit' } ){
       $self->{submitted}=1 ;
-      $self->{submit_value} = $name.'_uploaded';
+      $self->{submit_value} = $namew.'_uploaded';
+    }
+  }
+
+  if ($params->{type} eq 'Imagegalerie') {
+    my $f='';
+    $f = $self->find_action($name.'_delete_');
+    $self->{submit_value} = $namew.'_delete' if ($f ne '');
+    if ($f eq ''){
+      $f = $self->find_action($name.'_next_');
+      $self->{submit_value} = $namew.'_next' if ($f ne '');
+    }
+    if ($f eq ''){
+      $f = $self->find_action($name.'_prev_');
+      $self->{submit_value} = $namew.'_prev' if ($f ne '');
+    }
+    if ($f ne ''){
+      $self->{submitted}=1 ;
+      $self->{submit_id} = $f;
     }
   }
 
@@ -125,7 +164,6 @@ sub add_element{
     my $f='';
     $f = $self->find_action($name.'_delete_');
     if ($f ne ''){
-
       $self->{submitted}=1 ;
       $self->{submit_value} = $name.'_delete';
       $self->{submit_id} = $f;
@@ -135,7 +173,6 @@ sub add_element{
   if ($params->{type} eq 'Captcha') {
       my $tname=$name."_input";
       my $c_val = $self->get_value($name);
-
       $self->add_element({ type => 'Text',  name => $tname } );
       $self->add_constraint({ type=> 'Equation', operator=>'eq', name=>$tname, comp=>$c_val, text=>$params->{message} });
   }
@@ -157,7 +194,24 @@ sub find_action{
 
 sub do{
   my ($self, $name, $fn)=@_;
-  $self->{element}[$self->{element_index}->{$name}->{index}]->$fn();
+  $self->{element}[$self->{element_index}->{$self->{prefix}.$name}->{index}]->$fn();
+}
+
+sub get_javascript{
+  my ($self, $url)=@_;
+  my $js='';
+  my $result='';
+  my $usejquery = 0;
+  foreach my $item(@{$self->{element}}) {
+    if ($item->{js}){
+      $usejquery = 1;
+      $js.=$item->{js}."\n";
+    }
+  }
+  if ($usejquery==1){
+      $js='<script>'."\n".'$(document).ready(function(){ '.$js.' });'."\n".'</script>';
+  }
+  return $js;
 }
 
 sub get_jquery_modules{
@@ -201,37 +255,68 @@ sub get_jquery_modules{
   return $css_r.$result.$js.$self->{addition_modules};
 }
 
+sub set_table_class{
+  my ($self, $classname)=@_;
+  $self->{table_class}=$classname;
+}
+
+sub set_table_attributes{
+  my ($self, $attributes)=@_;
+
+  my $attr='';
+  while ( my ($key, $value) = each(%$attributes) ) {
+      $attr.=$key.'="'.$value.'" ';
+  }
+  $self->{table_attibutes}=$attr;
+}
+
 sub render{
-  my ($self, $view)=@_;
+  my ($self, $view, $action)=@_;
 
   my $table=-1;
   my $count=0;
 
-  my $result='<form method=post enctype="multipart/form-data">';
-    if ($view eq 'table'){ $result.='<table class="form_table"'; }
+  $action=' action="'.$action.'" ' if ($action);
+  $action='' if (!$action);
+  my $table_class='class="form_table"';
+  $table_class= 'class="'.$self->{table_class}.'"' if ($self->{table_class});
+  $table_class=$self->{table_attibutes} if ($self->{table_attibutes});
+
+    my $result='<form method=post '.$action.'enctype="multipart/form-data">';
+    if ($view eq 'table'){ $result.='<table '.$table_class.'>'; }
+
     foreach my $item(@{$self->{element}}) {
     my $name = $item->name;
 
     if ($self->{element_index}->{$name}->{ignore} ne 'true'){
       $item->{table}=-1;
 
-      if ($item->type eq "TableEnd") {
-         $item->{table}=-1;
-         $table=-1;
+      if ($view eq 'flat'){
+          if ($item->type ne 'Submit'){
+              my $label = $item->get_label();
+              my $value = $item->get_value();
+              $result.='<span class="form_label">'.$label."</span>: ".$value."<br />";
+          }
+      } else {
+          if ($item->type eq "TableEnd") {
+             $item->{table}=-1;
+             $table=-1;
+          }
+          if ($item->type eq "Table") {
+             $item->{table}=$item->columns;
+             $item->{colcount}=-1;
+             $count=-1;
+             $table=$item->columns;
+          }
+          if ($table>-1) {
+             $count++;
+             $count=1 if ($count>$table);
+             $item->{colcount}=$count;
+             $item->{table}=$table;
+          }
+          $result .= $item->render($self->{element_index}->{$name}, $view);
       }
-      if ($item->type eq "Table") {
-         $item->{table}=$item->columns;
-         $item->{colcount}=-1;
-         $count=-1;
-         $table=$item->columns;
-      }
-      if ($table>-1) {
-         $count++;
-         $count=1 if ($count>$table);
-         $item->{colcount}=$count;
-         $item->{table}=$table;
-      }
-      $result .= $item->render($self->{element_index}->{$name}, $view);
+
       }
   else {
      $result.="<input type='hidden' name='$name' value='".$item->get_value()."'>";
@@ -239,7 +324,7 @@ sub render{
 }
 
   if ($view eq 'table'){ $result.='</table>'; }
-  if ($view eq 'clean'){ }
+  #if ($view eq 'clean'){ }
   return $result.'</form>';
 }
 
@@ -258,6 +343,8 @@ sub submitted{
   my $set=0;
   if ($self->{submit_value} ne '') {
     $result=$self->{submit_value};
+    #$result=substr($result,length($self->{prefix})) if ($self->{prefix} ne'');
+
     foreach my $item(@{$self->{constraints}}) {
       my $name=$item->{name};
       if ($item->check() == 0){
@@ -272,7 +359,7 @@ sub submitted{
 
 sub get_single_dbix{
   my ($self,$name)=@_;
-  my $result = $self->{element}[$self->{element_index}->{$name}->{index}]->get_dbix();
+  my $result = $self->{element}[$self->{element_index}->{$self->{prefix}.$name}->{index}]->get_dbix();
   return $result;
 }
 
@@ -293,30 +380,29 @@ sub get_dbix{
 
 sub add_options{
   my ($self,$name,$options)=@_;
-  $self->{element}[$self->{element_index}->{$name}->{index}]->add_options($options);
+  $self->{element}[$self->{element_index}->{$self->{prefix}.$name}->{index}]->add_options($options);
 }
 
 sub reset_options{
   my ($self,$name,$options,$label,$id)=@_;
-
-  $self->{element}[$self->{element_index}->{$name}->{index}]->reset_options($options,$label,$id);
+  $self->{element}[$self->{element_index}->{$self->{prefix}.$name}->{index}]->reset_options($options,$label,$id);
 }
 
 sub freeze{
   my ($self, $name)=@_;
-  $self->{element_index}->{$name}->{frozen}=1;
-  $self->{element}[$self->{element_index}->{$name}->{index}]->freeze();
+  $self->{element_index}->{$self->{prefix}.$name}->{frozen}=1;
+  $self->{element}[$self->{element_index}->{$self->{prefix}.$name}->{index}]->freeze();
 }
 
 sub get_r{
   my ($self, $name)=@_;
-  $self->{element}[$self->{element_index}->{$name}->{index}]->pure(1) if (!$self->{element}[$self->{element_index}->{$name}->{index}]->pure);
-  return $self->{element}[$self->{element_index}->{$name}->{index}]->render();
+  $self->{element}[$self->{element_index}->{$self->{prefix}.$name}->{index}]->pure(1) if (!$self->{element}[$self->{element_index}->{$self->{prefix}.$name}->{index}]->pure);
+  return $self->{element}[$self->{element_index}->{$self->{prefix}.$name}->{index}]->render();
 }
 sub get_e{
   my ($self, $name)=@_;
-  return '' if (!$self->{element_index}->{$name}->{error_message});
-  return $self->{element_index}->{$name}->{error_message};
+  return '' if (!$self->{element_index}->{$self->{prefix}.$name}->{error_message});
+  return $self->{element_index}->{$self->{prefix}.$name}->{error_message};
 }
 
 sub get_errors{
@@ -324,7 +410,7 @@ sub get_errors{
   my $k;
   my $result='';
   foreach $k(keys %{ $self->{element_index} } ){
-    $result.=$self->{element_index}->{$k}->{error_message}.</ br> if ( $self->{element_index}->{$k}->{error_message});
+    $result.=$self->{element_index}->{$k}->{error_message}.'<br />' if ( $self->{element_index}->{$k}->{error_message});
   }
   return $result;
 }
@@ -340,38 +426,77 @@ sub freeze_all{
 
 sub unfreeze{
   my ($self, $name)=@_;
-  $self->{element_index}->{$name}->{frozen}=0;
+  $self->{element_index}->{$self->{prefix}.$name}->{frozen}=0;
 }
 
 sub get_value{
   my ($self, $name)=@_;
   my $result='';
-  $result=$self->{element}[$self->{element_index}->{$name}->{index}]->get_value();
+  $result=$self->{element}[$self->{element_index}->{$self->{prefix}.$name}->{index}]->get_value();
   return $result;
 }
 
 sub populate{
-  my ($self, $data)=@_;
-  my @columns= $data->result_source->columns;
+  my ($self, $data, $anyway)=@_;
 
-  foreach my $item(keys %{$self->{element_index}}) {
-    if ( grep { $item eq $_ } @columns ) {
-       if (!$self->{request}->{$item}) { $self->{request}->{$item}=$data->$item;  }
+  if (($self->{submit_value} eq '') or ($anyway ne '')) {
+    if (ref($data) eq 'HASH') {
+      while (my ($key, $value) = each %{ $data }){
+             $self->{request}->{$self->{prefix}.$key}=$value;
+      }
+    } else {
+      my @columns= $data->result_source->columns;
+
+      foreach my $item(keys %{$self->{element_index}}) {
+        $item=substr($item,length($self->{prefix})) if ($self->{prefix} ne'');
+        if ( grep { $item eq $_ } @columns ) {
+           if (!$self->{request}->{$self->{prefix}.$item}) {
+            $self->{request}->{$self->{prefix}.$item}=$data->get_column($item);
+           }
+        }
+      }
     }
   }
 }
 
+sub serial_populate{
+  my ($self, $data)=@_;
+  my $result = {};
+  my @arr_data = split('&',$data);
+  foreach (@arr_data) {
+     my @tmp = split('=',$_);
+     $self->{request}->{$self->{prefix}.$tmp[0]} = $tmp[1] if ($tmp[1]);
+  }
+}
+
 sub map_value{
   my ($self, @columns)=@_;
   my $result;
-
   foreach my $item(keys %{$self->{element_index}}) {
-    if ( grep { $item eq $_ } @columns ) {
-    	$result->{$item}=$self->get_value($item); }
+    $item=substr($item,length($self->{prefix})) if ($self->{prefix} ne'');
+    my $type=$self->{element}[$self->{element_index}->{$self->{prefix}.$item}->{index}]->type;
+    if (($type ne 'Upload')&&($type ne 'Image')){
+      if ( grep { $item eq $_ } @columns ) {    
+          $result->{$item}=$self->get_value($item);
+      }
+    }
   }
  return $result;
 }
 
+
+sub get_values{
+  my ($self)=@_;
+  my $result;
+
+  foreach my $item(keys %{$self->{element_index}}) {
+    $item=substr($item,length($self->{prefix})) if ($self->{prefix} ne'');
+    $result->{$item}=$self->get_value($item);
+  }
+ return $result;
+}
+
+
 1;
 
 __END__
@@ -605,7 +730,7 @@ Expects an array with column names. This method is used to map the request and f
 
 =head1 AUTHOR
 
-Thorsten Domsch, camelcase@gmx.de
+Thorsten Drobnik, camelcase@hotmail.com
 
 =head1 LICENSE
 
@@ -613,5 +738,3 @@ This library is free software, you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
 =cut
-
-
